searchData={"items":[{"type":"module","title":"RequestCache","doc":"","ref":"RequestCache.html"},{"type":"module","title":"RequestCache - RequestCache","doc":"[![Test](https://github.com/MikaAK/request_cache_plug/actions/workflows/test-actions.yml/badge.svg)](https://github.com/MikaAK/request_cache_plug/actions/workflows/test-actions.yml)\n[![codecov](https://codecov.io/gh/MikaAK/request_cache_plug/branch/main/graph/badge.svg?token=RF4ASVG5PV)](https://codecov.io/gh/MikaAK/request_cache_plug)\n[![Hex version badge](https://img.shields.io/hexpm/v/request_cache_plug.svg)](https://hex.pm/packages/request_cache_plug)\n\nThis plug allows us to cache our graphql queries and phoenix controller requests declaritevly\n\nWe call the cache inside either a resolver or a controller action and this will store it preventing further\nexecutions of our query on repeat requests.\n\nThe goal of this plug is to short-circuit any processing phoenix would\nnormally do upon request including json decoding/parsing, the only step that should run is telemetry","ref":"RequestCache.html#module-requestcache"},{"type":"module","title":"Installation - RequestCache","doc":"This  package can be installed by adding `request_cache_plug` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:request_cache_plug, \"~> 1.0\"}\n  ]\nend\n```\n\nDocumentation can be found at  .","ref":"RequestCache.html#module-installation"},{"type":"module","title":"Config - RequestCache","doc":"This is the default config, it can all be changed\n```elixir\nconfig :request_cache_plug,\n  enabled?: true,\n  verbose?: false,\n  graphql_paths: [\"/graphiql\", \"/graphql\"],\n  conn_priv_key: :__shared_request_cache__,\n  request_cache_module: RequestCache.ConCacheStore,\n  default_ttl: :timer.hours(1),\n  default_concache_opts: [\n    ttl_check_interval: :timer.seconds(1),\n    acquire_lock_timeout: :timer.seconds(1),\n    ets_options: [write_concurrency: true, read_concurrency: true]\n  ]\n```","ref":"RequestCache.html#module-config"},{"type":"module","title":"Usage - RequestCache","doc":"This plug is intended to be inserted into the `endpoint.ex` fairly early in the pipeline,\nit should go after telemetry but before our parsers\n\n```elixir\nplug Plug.Telemetry, event_prefix: [:phoenix, :endpoint]\n\nplug RequestCache.Plug\n\nplug Plug.Parsers,\n  parsers: [:urlencoded, :multipart, :json],\n  pass: [\"*/*\"]\n```\n\nWe also need to setup a before_send hook to our absinthe_plug (if not using absinthe you can skip this step)\n```elixir\nplug Absinthe.Plug, before_send: {RequestCache, :connect_absinthe_context_to_conn}\n```\nWhat this does is allow us to see the results of items we put onto our request context from within plugs coming after absinthe\n\nAfter that we can utilize our cache in a few ways\n\n#### Utilization with Phoenix Controllers\n```elixir\ndef index(conn, params) do\n  conn\n    |> RequestCache.store(:timer.seconds(60))\n    |> put_status(200)\n    |> json(%{...})\nend\n```\n\n#### Utilization with Absinthe Resolvers\n```elixir\ndef all(params, _resolution) do\n  # Instead of returning {:ok, value} we return this\n  RequestCache.store(value, :timer.seconds(60))\nend\n```\n\n#### Utilization with Absinthe Middleware\n```elixir\nfield :user, :user do\n  arg :id, non_null(:id)\n\n  middleware RequestCache.Middleware, ttl: :timer.seconds(60)\n\n  resolve &Resolvers.User.find/2\nend\n```","ref":"RequestCache.html#module-usage"},{"type":"module","title":"Specifying Specific Caching Locations - RequestCache","doc":"We have a few ways to control the caching location of our RequestCache, by default if you have `con_cache` installed,\nwe have access to `RequestCache.ConCacheStore` which is the default setting\nHowever we can override this by setting `config :request_cache_plug, :request_cache_module, MyCustomCache`\n\nCaching module will be expected to have the following API:\n```elixir\ndef get(key) do\n  ...\nend\n\ndef put(key, ttl, value) do\n  ...\nend\n```\n\nYou are responsible for starting the cache, including ConCacheStore, so if you're planning to use it make sure\nyou add `RequestCache.ConCacheStore` to the application.ex list of children\n\n***Specifying the module per function is currently not fully implemented, check back soon for updates***\n\nWe can also override the module for a particular request by passing the option to our graphql middleware or\nour `&RequestCache.store/2` function as `[ttl: 123, cache: MyCacheModule]`\n\n##### With Middleware\n\n```elixir\nfield :user, :user do\n  arg :id, non_null(:id)\n\n  middleware RequestCache.Middleware, ttl: :timer.seconds(60), cache: MyCacheModule\n\n  resolve &Resolvers.User.find/2\nend\n```\n\n##### In a Resolver\n\n```elixir\ndef all(params, resolution) do\n  RequestCache.store(value, ttl: :timer.seconds(60), cache: MyCacheModule)\nend\n```\n\n##### In a Controller\n\n```elixir\ndef index(conn, params) do\n  RequestCache.store(conn, ttl: :timer.seconds(60), cache: MyCacheModule)\nend\n```","ref":"RequestCache.html#module-specifying-specific-caching-locations"},{"type":"module","title":"telemetry - RequestCache","doc":"Cache events are emitted via :telemetry. Events are:\n\n- `[:request_cache_plug, :graphql, :cache_hit]`\n- `[:request_cache_plug, :graphql, :cache_miss]`\n- `[:request_cache_plug, :rest, :cache_hit]`\n- `[:request_cache_plug, :rest, :cache_miss]`\n- `[:request_cache_plug, :cache_put]`\n\nFor GraphQL endpoints it is possible to provide a list of atoms that will be passed through to the event metadata; e.g.:\n\n##### With Middleware\n\n```elixir\nfield :user, :user do\n  arg :id, non_null(:id)\n\n  middleware RequestCache.Middleware,\n    ttl: :timer.seconds(60),\n    cache: MyCacheModule,\n    labels: [:service, :endpoint],\n    whitelisted_query_names: [\"MyQueryName\"] # By default all queries are cached, can also whitelist based off query name from GQL Document\n\n  resolve &Resolvers.User.find/2\nend\n```\n\n##### In a Resolver\n\n```elixir\ndef all(params, resolution) do\n  RequestCache.store(value, ttl: :timer.seconds(60), cache: MyCacheModule, labels: [:service, :endpoint])\nend\n```\n\nThe events will look like this:\n\n```elixir\n{\n  [:request_cache_plug, :graphql, :cache_hit],\n  %{count: 1},\n  %{ttl: 3600000, cache_key: \"/graphql:NNNN\", labels: [:service, :endpoint]}\n}\n```\n\n##### Enable Error Caching\nIn order to enable error caching we can either setup `cached_errors` in our config\nor as an option to `RequestCache.store` or `RequestCache.Middleware`.\n\nThe value of `cached_errors` can be one of `[]`, `:all` or a list of reason_atoms as\ndefined by `Plug.Conn.Status` such as `:not_found`, or `:internal_server_error`.\n\nIn REST this works off the response codes returned. However, in order to use reason_atoms in GraphQL\nyou will need to make sure your errors contain some sort of `%{code: \"not_found\"}` response in them\n\nTake a look at [error_message](https://github.com/MikaAK/elixir_error_message) for a compatible error system","ref":"RequestCache.html#module-telemetry"},{"type":"module","title":"Notes/Gotchas - RequestCache","doc":"- In order for this caching to work, we cannot be using POST requests as specced out by GraphQL, not for queries at least, fortunately this doesn't actually matter since we can use any http method we want (there will be a limit to query size), in a production app you may be doing this already due to the caching you gain from CloudFlare\n- Caches are stored via a MD5 hashed key that correlates to your query in GraphQL, or in REST your url path + query parameters\n- Absinthe and ConCache are optional dependencies, if you don't have them you won't have access to `RequestCache.Middleware` or `RequestCache.ConCacheStore`\n- If no ConCache is found, you must set `config :request_cache_module` to something else","ref":"RequestCache.html#module-notes-gotchas"},{"type":"module","title":"Caching Header - RequestCache","doc":"When an item is served from the cache, we return a header `rc-cache-status` which has a value of `HIT`. Using this you can tell if the item was\nserved out of cache, without it the item was fetched","ref":"RequestCache.html#module-caching-header"},{"type":"module","title":"Example Reduction - RequestCache","doc":"In the case of a large (16mb) payload running through absinthe, this plug cuts down response times from 400+ms -> <400μs","ref":"RequestCache.html#module-example-reduction"},{"type":"function","title":"RequestCache.connect_absinthe_context_to_conn/2","doc":"","ref":"RequestCache.html#connect_absinthe_context_to_conn/2"},{"type":"function","title":"RequestCache.store/2","doc":"","ref":"RequestCache.html#store/2"},{"type":"type","title":"RequestCache.opts/0","doc":"","ref":"RequestCache.html#t:opts/0"},{"type":"module","title":"RequestCache.Middleware","doc":"","ref":"RequestCache.Middleware.html"},{"type":"function","title":"RequestCache.Middleware.store_result/2","doc":"","ref":"RequestCache.Middleware.html#store_result/2"},{"type":"module","title":"RequestCache.Plug","doc":"This plug allows you to cache GraphQL requests based off their query name and\nvariables. This should be placed right after telemetry and before parsers so that it can\nstop any processing of the requests and immediately return a response.\n\nPlease see `RequestCache` for more details","ref":"RequestCache.Plug.html"},{"type":"function","title":"RequestCache.Plug.empty_errors_pattern/0","doc":"","ref":"RequestCache.Plug.html#empty_errors_pattern/0"},{"type":"function","title":"RequestCache.Plug.error_codes_pattern/1","doc":"","ref":"RequestCache.Plug.html#error_codes_pattern/1"},{"type":"function","title":"RequestCache.Plug.error_pattern/0","doc":"","ref":"RequestCache.Plug.html#error_pattern/0"},{"type":"function","title":"RequestCache.Plug.request_cache_header/0","doc":"","ref":"RequestCache.Plug.html#request_cache_header/0"},{"type":"function","title":"RequestCache.Plug.request_cache_key_header/0","doc":"","ref":"RequestCache.Plug.html#request_cache_key_header/0"},{"type":"function","title":"RequestCache.Plug.store_request/2","doc":"","ref":"RequestCache.Plug.html#store_request/2"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}